-- AnnoyingBuffReminder Revived - by Kashi (kashi1988@gmail.com)
-- -- original Author: Wikwocket (wikwocket@gmail.com)
-- Displays an annoying reminder to use armor spells, auras, shouts, etc, and Victory Rush.

-- TODO
-- shaman
-- hunter
-- paladin
-- rogue
-- reminders depending on talents (f.e. warlock-sacrifice)
-- most level requirements
-- redesign ui

local VERSION = "r4";

-- Print: helper function to send message to default chat frame
local function Print(text)
	if (DEFAULT_CHAT_FRAME) then
		DEFAULT_CHAT_FRAME:AddMessage(text);
	end
end

local className = nil;
local inCombat = false;
local spec = GetSpecialization();
local level = UnitLevel("player");
local stance = GetShapeshiftFormID();


-- OnLoad: register events, slash commands, and hooks, print welcome message
function ABRRevivedOnLoad()
	ABRRevivedHoverFrame:Hide();
	_, className = UnitClass("player");
	SlashCmdList["ABRRevived"] = ABRRevivedSlashCommand;
	SLASH_ABRRevived1 = "/ABRRevived";
	SLASH_ABRRevived2 = "/abrr";
	
	ABRRevivedHoverFrame:RegisterEvent("VARIABLES_LOADED");
	
	ABRRevivedHoverFrame:RegisterEvent("UNIT_AURA");
	ABRRevivedHoverFrame:RegisterEvent("UNIT_INVENTORY_CHANGED");
	ABRRevivedHoverFrame:RegisterEvent("PLAYER_ENTERING_WORLD");
	
	ABRRevivedHoverFrame:RegisterEvent("PLAYER_REGEN_DISABLED");
	ABRRevivedHoverFrame:RegisterEvent("PLAYER_REGEN_ENABLED");
	ABRRevivedHoverFrame:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED");
	ABRRevivedHoverFrame:RegisterEvent("PLAYER_LEVEL_UP");
	ABRRevivedHoverFrame:RegisterEvent("UPDATE_SHAPESHIFT_FORM");
end

-- SlashCommand: handle slash commands
function ABRRevivedSlashCommand(command)
	if (command == "lock") then
		ABRRevivedVariables.lock = true;
		ABRRevivedHoverFrame:EnableMouse(false);
		Print("ABRRevived: Window locked. You can now click through the reminder window.");
	-- unlock frame
	elseif (command == "unlock") then
		ABRRevivedVariables.lock = false;
		ABRRevivedHoverFrame:EnableMouse(true);
		Print("ABRRevived: Window unlocked. You can now move the reminder window by clicking and dragging.");
	-- re-center frame
	elseif (command == "reset" or command == "center") then
		ABRRevivedHoverFrame:ClearAllPoints();
		ABRRevivedHoverFrame:SetPoint("CENTER", UIParent, "CENTER", 0, 0);
		Print("ABRRevived: Window position reset to center of screen.");
	-- toggle spell icon display
	elseif (command == "icon" or command == "icons") then
		ABRRevivedVariables.dontShowIcons = not ABRRevivedVariables.dontShowIcons;
		if (ABRRevivedVariables.dontShowIcons) then
			Print("ABRRevived: Spell icons will no longer be shown.");
		else
			Print("ABRRevived: Spell icons will now be shown with spell names.");
		end
	-- buff list
	elseif (command == "stancer") then
		ABRRevivedVariables.stancer = (not ABRRevivedVariables.stancer);
		if (ABRRevivedVariables.stancer) then
			Print("ABRRevived: Stancer activated.");
		else
			Print("ABRRevived: Stancer deactivated.");
		end
		
	-- -- debug / test commands
	-- display spec + level
	elseif (command == "test") then
		Print("ID of current spec: " .. spec);
		Print("Current Level: " .. level);
	-- stance
	elseif (command == "stance") then
		if (stance ~= nil) then
			Print("ID of current stance: " .. stance);
		else
			Print("No stance!");
		end
	-- buff list
	elseif (command == "buffs") then
		for i = 1, 40 do
			buffName = UnitAura("player", i, "HELPFUL");
			Print("buff " .. i .. " is " .. tostring(buffName));
		end
		
	-- default: print usage menu
	else
		Print("ABRRevived " .. VERSION);
		Print("Displays an annoying reminder to use buffs, armor spells, etc.");
		Print("  /abrr icons - Toggle display of spell icons");
		Print("  /abrr lock||unlock - Toggle window locking and click-through");
		Print("  /abrr center - Reset window position to center of screen");
		Print("  /abrr stancer - Toggle stance/presence helper functionality");
	end
end

-- OnEvent:
function ABRRevivedOnEvent(self, event, ...)
	local arg1 = select(1, ...);
	-- VARIABLES_LOADED: initialize
	if (event == "VARIABLES_LOADED") then
		-- load (or create) saved variables
		if (not ABRRevivedVariables) then
			ABRRevivedVariables = {};
			ABRRevivedVariables.lock = false;
			ABRRevivedVariables.stancer = true;
			-- let other vars init to nil/false
		end
		-- lock or unlock
		ABRRevivedHoverFrame:EnableMouse(not ABRRevivedVariables.lock);
		return;
	end
	
	-- PLAYER_REGEN_DISABLED: set combat flag
	if (event == "PLAYER_REGEN_DISABLED") then
		inCombat = true;
	end
	
	-- PLAYER_REGEN_ENABLED: clear combat flag
	if (event == "PLAYER_REGEN_ENABLED") then
		inCombat = false;
	end
	
	-- PLAYER_SPECIALIZATION_CHANGED: spec change -> update spec
	if (event == "PLAYER_SPECIALIZATION_CHANGED") then
		spec = GetSpecialization();
	end
	
	-- PLAYER_LEVEL_UP: levelup -> update level
	if (event == "PLAYER_LEVEL_UP") then
		level = UnitLevel("player");
	end
	
	-- UPDATE_SHAPESHIFT_FORM: levelup -> update level
	if (event == "UPDATE_SHAPESHIFT_FORM") then
		stance = GetShapeshiftFormID();
	end
	
	-- abort if inappropriate to notify player now
	if (IsFlying() or UnitOnTaxi("player") or IsResting() or not HasFullControl()) then
			ABRRevivedHoverFrame:Hide();
			return;
	end
	
	-- vars for whether ABRR should be shown, used in code blocks below
	local showABRR = false;
	local textABRR = "";
	local spellABRR = ""; -- used for determining spell icon texture
	local spellABRRalt = ""; -- used for determining 2nd spell icon texture
	local buffName;
	local casterID; -- note: only used for classes for whom all of the buffs we check are self-cast only
	
	-- UNIT_AURA: 										check for appropriate armor spells/etc
	-- UPDATE_SHAPESHIFT_FORM: 				stance dance
	-- PLAYER_SPECIALIZATION_CHANGED:	switch spec
	-- PLAYER_LEVEL_UP:								leve up
	if (not(event == "UNIT_AURA" and arg1 ~= "player")) then
		if (className == "PRIEST") then
			-- inner fire, pw:fort
			local needI = true;
			local needPWF = true;
			local needSF = true;
			for i = 1, 40 do
				buffName = UnitAura("player", i, "HELPFUL");
				-- Print("buff " .. i .. " is " .. tostring(buffName) .. "/" .. tostring(casterID));
				if (buffName) then
					if (buffName == ABRR_LOCALE.innerFire
						or buffName == ABRR_LOCALE.innerWill) then
						needI = false;
					elseif (buffName == ABRR_LOCALE.powerFortitude
						or buffName == ABRR_LOCALE.darkIntent
						or buffName == ABRR_LOCALE.songOfVictory) then
							needPWF = false;
					elseif (buffName == ABRR_LOCALE.shadowform) then
							needSF = false;
					end
				end
			end
			if (needI) then
				showABRR = true;
				textABRR = "Cast Inner Fire/Will!";
				spellABRR = ABRR_LOCALE.innerFire;
				spellABRRalt = ABRR_LOCALE.innerWill;
			elseif (needPWF) then
				showABRR = true;
				textABRR = "Cast PW:Fortitude!";
				spellABRR = ABRR_LOCALE.powerFortitude;
			elseif (needSF and spec == 3) then
				showABRR = true;
				textABRR = "Shadowform!";
				spellABRR = ABRR_LOCALE.shadowform;
			end
		elseif (className == "WARLOCK") then
			--  dark intent
			local needDI = true;
			for i = 1, 40 do
				buffName = UnitAura("player", i, "HELPFUL");
				if (buffName) then
					if (buffName == ABRR_LOCALE.powerFortitude
						or buffName == ABRR_LOCALE.arcaneBrilliance
						or buffName == ABRR_LOCALE.dalaranBrilliance
						or buffName == ABRR_LOCALE.darkIntent) then
							needDI = false;
					end
				end
			end
			if (needDI and level >= 82) then
				showABRR = true;
				textABRR = "Cast an Dark Intent!";
				spellABRR = ABRR_LOCALE.darkIntent;
			end
		elseif (className == "MAGE") then
			-- armor spell, ai
			local needArmor = true;
			local needAI = true;
			for i = 1, 40 do
				buffName = UnitAura("player", i, "HELPFUL");
				if (buffName) then
					if (buffName == ABRR_LOCALE.frostArmor
						or buffName == ABRR_LOCALE.mageArmor
						or buffName == ABRR_LOCALE.moltenArmor) then
							needArmor = false;
					elseif (buffName == ABRR_LOCALE.arcaneBrilliance
						or buffName == ABRR_LOCALE.dalaranBrilliance
						or buffName == ABRR_LOCALE.songOfVictory) then
							needAI = false;
					end
				end
			end
			if (needArmor) then
				showABRR = true;
				textABRR = "Cast an Armor!";
				spellABRR = ABRR_LOCALE.moltenArmor;
				spellABRRalt = ABRR_LOCALE.frostArmor;
			elseif (needAI) then
				showABRR = true;
				textABRR = "Cast Arcane Int!";
				spellABRR = ABRR_LOCALE.arcaneBrilliance;
			end
		elseif (className == "SHAMAN") then
			-- main hand weapon enchant
			local mainHandEnchanted, _, _, offHandEnchanted = GetWeaponEnchantInfo();
			if (not mainHandEnchanted) then
				showABRR = true;
				textABRR = "Enchant Main Hand!";
				if (spec == 1) then
					spellABRR = ABRR_LOCALE.flametongueWeapon;
				elseif (spec == 2) then
					spellABRR = ABRR_LOCALE.windfuryWeapon;
				elseif (spec == 3) then
					spellABRR = ABRR_LOCALE.earthlivingWeapon;
				end
			end
			-- off hand weapon enchant
			if (not showABRR) then
				local offHandLink = GetInventoryItemLink("player", 17);
				if (offHandLink) then
					local _, _, _, _, _, itemType = GetItemInfo(offHandLink);
					if (itemType == ABRR_LOCALE.itemTypeWeapon and not offHandEnchanted) then
						showABRR = true;
						textABRR = "Enchant Off Hand!";
						spellABRR = ABRR_LOCALE.flametongueWeapon;
					end
				end
			end
			-- shield spell
			if (not showABRR) then
				local needShield = true;
				for i = 1, 40 do
					-- use PLAYER filter and casterID since only need to track *player*-cast spells
					buffName, _, _, _, _, _, _, casterID, _ = UnitAura("player", i, "HELPFUL|PLAYER");
					if (buffName and casterID == "player") then
						if ((buffName == ABRR_LOCALE.lightningShield and (spec == 1 or spec == 2))
							or ((buffName == ABRR_LOCALE.waterShield
							or buffName == ABRR_LOCALE.earthShield) and spec == 3)) then
								needShield = false;
						end
					end
				end
				if (needShield) then
					showABRR = true;
					textABRR = "Cast a Shield!";
					if (spec == 1 or spec == 2) then
						spellABRR = ABRR_LOCALE.lightningShield;
					elseif (spec == 3) then
						spellABRR = ABRR_LOCALE.waterShield;
					end
				end
			end
		elseif (className == "HUNTER") then
			-- aspect
			local needAspect = true;
			for i = 1, 40 do
				-- use PLAYER filter and casterID since only need to track *player*-cast spells
				buffName, _, _, _, _, _, _, casterID, _ = UnitAura("player", i, "HELPFUL|PLAYER");
				if (buffName and casterID == "player") then
					if (string.match(buffName, ABRR_LOCALE.regexAspect)) then
						needAspect = false;
					end
					if (inCombat and (buffName == ABRR_LOCALE.aspectPack or buffName == ABRR_LOCALE.aspectCheetah)) then
						needAspect = true;
					end
				end
			end
			if (needAspect) then
				showABRR = true;
				textABRR = "Cast an Aspect!";
				spellABRR = ABRR_LOCALE.aspectHawk;
			end
		elseif (className == "ROGUE") then
			-- poisons
			local needDPSPoison = true;
			local needSlowPoison = true;
			for i = 1, 40 do
				-- use PLAYER filter and casterID since only need to track *player*-cast spells
				buffName, _, _, _, _, _, _, casterID, _ = UnitAura("player", i, "HELPFUL|PLAYER");
				if (buffName and casterID == "player") then
					if (buffName == ABRR_LOCALE.mindPoison or buffName == ABRR_LOCALE.slowPoison) then
						needSlowPoison = false;
					elseif (buffName == ABRR_LOCALE.msPoison or buffName == ABRR_LOCALE.deadlyPoison) then
						needDPSPoison = false;
					end
				end
			end
			if (needSlowPoison) then
				showABRR = true;
				textABRR = "Apply Slow-Poison!";
				spellABRR = ABRR_LOCALE.mindPoison;
				spellABRRalt = ABRR_LOCALE.slowPoison;
			elseif (needDPSPoison) then
				showABRR = true;
				textABRR = "Apply DPS-Poison!";
				spellABRR = ABRR_LOCALE.deadlyPoison;
				spellABRRalt = ABRR_LOCALE.msPoison;
			end
		elseif (className == "PALADIN") then
			-- blessingKings, blessingMight
			local needKings = true;
			local needMight = true;
			for i = 1, 40 do
				buffName, _, _, _, _, _, _, casterID, _ = UnitAura("player", i, "HELPFUL");
				if (buffName) then
					if (buffName == ABRR_LOCALE.markWild
						or buffName == ABRR_LOCALE.blessingKings
						or buffName == ABRR_LOCALE.legacyEmperor
						or (buffName == ABRR_LOCALE.blessingMight and casterID == "player")) then
						needKings = false;
					end
					if ((buffName == ABRR_LOCALE.blessingKings and casterID == "player")
						or buffName == ABRR_LOCALE.blessingMight) then
						needMight = false;
					-- elseif (string.match(buffName, ABRR_LOCALE.regexSeal)) then
					--	needSeal = false;
					end
				end
			end
			if (needKings) then
				showABRR = true;
				textABRR = "Cast a Blessing of Kings!";
				spellABRR = ABRR_LOCALE.blessingKings;
			elseif (needMight) then
				showABRR = true;
				textABRR = "Cast a Blessing of Might!";
				spellABRR = ABRR_LOCALE.blessingMight;
			end
		elseif (className == "DRUID") then
			-- motw
			local needMark = true;
			for i = 1, 40 do
				buffName = UnitAura("player", i, "HELPFUL");
				if (buffName) then
					if (buffName == ABRR_LOCALE.markWild
						or buffName == ABRR_LOCALE.blessingKings
						or buffName == ABRR_LOCALE.legacyEmperor) then
						needMark = false;
					end
				end
			end
			if (needMark) then
				showABRR = true;
				textABRR = "Cast Mark of the Wild!";
				spellABRR = ABRR_LOCALE.markWild;
			end
		elseif (className == "DEATHKNIGHT") then
			-- presence, horn of winter, bone shield
			local presence = "none";
			for i = 1, 40 do
				buffName = UnitAura("player", i, "HELPFUL");
				if (buffName) then
					if (buffName == ABRR_LOCALE.bloodPresence) then
						presence = "blood";
					elseif (buffName == ABRR_LOCALE.frostPresence) then
						presence = "frost";
					elseif (buffName == ABRR_LOCALE.unholyPresence) then
						presence = "unholy";
					end
				end
			end
			if (presence ~= "none") then
				if ((spec == 2 or spec == 3) and presence == "blood" and ABRRevivedVariables.stancer) then
					showABRR = true;
					textABRR = "Switch Presence!";
					spellABRR = ABRR_LOCALE.frostPresence;
					spellABRRalt = ABRR_LOCALE.unholyPresence;
				elseif (spec == 1 and (presence == "frost" or presence == "unholy") and ABRRevivedVariables.stancer) then
					showABRR = true;
					textABRR = "Switch Presence!";
					spellABRR = ABRR_LOCALE.bloodPresence;
				end
			else
				showABRR = true;
				textABRR = "Activate Presence!";
				if ((spec == 2 or spec == 3) and ABRRevivedVariables.stancer) then
					spellABRR = ABRR_LOCALE.frostPresence;
					spellABRRalt = ABRR_LOCALE.unholyPresence;
				elseif (spec == 1 and ABRRevivedVariables.stancer) then
					spellABRR = ABRR_LOCALE.bloodPresence;
				end
			end
		elseif (className == "MONK") then
			-- legacyEmperor, legacyWhiteTiger
			local needEmperor = true;
			local needWhiteTiger = true;
			for i = 1, 40 do
				buffName = UnitAura("player", i, "HELPFUL");
				if (buffName) then
					if (buffName == ABRR_LOCALE.markWild
						or buffName == ABRR_LOCALE.blessingKings
						or buffName == ABRR_LOCALE.legacyEmperor) then
						needEmperor = false;
					elseif (buffName == ABRR_LOCALE.legacyWhiteTiger) then
						needWhiteTiger = false;
					end
				end
			end
			if (needEmperor) then
				showABRR = true;
				textABRR = "Cast Legacy of the Emperor!";
				spellABRR = ABRR_LOCALE.legacyEmperor;
			elseif (needWhiteTiger and spec == 3 and level >= 81) then
				showABRR = true;
				textABRR = "Cast Legacy of the White Tiger!";
				spellABRR = ABRR_LOCALE.legacyWhiteTiger;
			end
		elseif (className == "WARRIOR") then
			if ((spec == 1 or spec == 2) and stance == 18 and ABRRevivedVariables.stancer) then
				showABRR = true;
				textABRR = "Switch Stance!";
				spellABRR = ABRR_LOCALE.stanceBattle17;
				spellABRRalt = ABRR_LOCALE.stanceBerserk19;
			elseif (spec == 3 and (stance == 17 or stance == 19) and ABRRevivedVariables.stancer) then
				showABRR = true;
				textABRR = "Switch Stance!";
				spellABRR = ABRR_LOCALE.stanceDef18;
			end
		end -- end classes ifelseif
		
		-- display or hide ABRR as needed
		if (showABRR)
			--and((UnitPower("player", SPELL_POWER_MANA) * 100 / UnitPowerMax("player", SPELL_POWER_MANA) >= 50) -- if >= 50% mana
			--or (className == "SHAMAN" and textABRR == "Cast a Shield!") -- or a shaman showing a no-mana-cost shield reminder
			--or (className == "MONK" or className == "WARRIOR" or className == "ROGUE" or className == "DEATHKNIGHT" or className == "HUNTER"))
				then
			ABRRevivedText:SetText(textABRR);
			ABRRevivedHoverFrame:Show();
			if (ABRRevivedVariables.dontShowIcons) then
				ABRRevivedSpellButton1Icon:Hide();
				ABRRevivedSpellButton2Icon:Hide();
				ABRRevivedHoverFrame:SetWidth(200);
			else
				if (spellABRRalt == "") then
					local texture = select(3, GetSpellInfo(spellABRR));
					ABRRevivedSpellButton1Icon:SetTexture(texture);
					ABRRevivedSpellButton2Icon:SetTexture(texture);
					ABRRevivedSpellButton1Icon:Show();
					ABRRevivedSpellButton2Icon:Show();
					ABRRevivedHoverFrame:SetWidth(265);
				else
					local texture = select(3, GetSpellInfo(spellABRR));
					local texturealt = select(3, GetSpellInfo(spellABRRalt));
					ABRRevivedSpellButton1Icon:SetTexture(texture);
					ABRRevivedSpellButton2Icon:SetTexture(texturealt);
					ABRRevivedSpellButton1Icon:Show();
					ABRRevivedSpellButton2Icon:Show();
					ABRRevivedHoverFrame:SetWidth(265);
				end	
			end
		else
			ABRRevivedHoverFrame:Hide();
		end
		return;
	end -- end UNIT_MANA/UNIT_AURA if
end
